/* jshint esversion : 6 */
/* jshint ignore : start */

const gulp = require('gulp');
const browserify = require('browserify');
const babel = require('babelify');
//const babel = require('gulp-babel');
const clean = require('gulp-clean');
const del = require('del');
const concat = require('gulp-concat');
const source = require('vinyl-source-stream');
const buffer = require('vinyl-buffer');
const { readdirSync, readFileSync, writeFileSync, unlinkSync, writeFile, lstatSync } = require('fs');
const path = require('path');




const babelPolyfill = path.join(__dirname, '.', 'node_modules', '.', 'babel-polyfill', '.', 'dist', '.', 'polyfill.min.js');
const indexeddbshim = path.join(__dirname, '.', 'node_modules', '.', 'indexeddbshim', '.', 'dist', '.', 'indexeddbshim-UnicodeIdentifiers.min.js');


function buildWorkerPolyfill() {
    const dest = path.join(__dirname, '.', 'src', '.', 'workers', '.', 'assets');
    return gulp.src(babelPolyfill)
        .pipe(gulp.dest(dest));
    //.pipe(del(preBuilt));
}

function buildIDBShim() {
    const dest = path.join(__dirname, '.', 'src', '.', 'workers', '.', 'assets');
    return gulp.src(indexeddbshim)
        .pipe(gulp.dest(dest));
    //.pipe(del(preBuilt));
}

function buildIndex() {
    const entries = path.join(__dirname, '.', 'src', '.', 'AppIDB.js');
    return browserify({ entries })
        .transform(babel)
        .bundle()
        .on('error', err => console.error(err))
        .pipe(source('index.js'))
        .pipe(buffer())
        .pipe(gulp.dest(__dirname))
}



function buildQueries() {
    //
    const src = path.join(__dirname, '.', 'src', '.', 'queries');
    const dest = path.join(__dirname, '.', 'src', '.', 'workers', '.', 'assets');


    function initBuild() {
        return new Promise(resolve => {
            let content = '/* jshint esversion : 6 */\n';
            // content += 'require("@babel/plugin-transform-runtime");\n';
            content += `const $idb = require('../IDB/index');\n`

            const files = readdirSync(src).filter(e => {
                if (/\.js$/.test(e) && 'index.js' !== e)
                    return e;
            }).map(e => {

                const key = e.replace('__', '').replace(/\.js$/, '');
                const val = `require('./${e}')`;

                return `exports.${key} = ${val}($idb);`;
            }).join('\n');


            content += `${files}\nself.$queries = exports;`;


            try {
                unlinkSync(`${src}/index.js`);
            } catch (e) {
                setTimeout(Function.prototype, 10000);
            }
            writeFile(`${src}/index.js`, content, function() { resolve(); });
        });
    }


    const entries = path.join(src, '.', 'index.js');


    function build() {
        return browserify(entries)
            .transform(babel)
            .bundle()
            .on('error', err => console.error(err))
            .pipe(source(`$queries.js`))
            .pipe(buffer())
            .pipe(gulp.dest(dest))
    }

    gulp.task(`build_$queries`, gulp.series(initBuild, build), () => Promise.resolve());

}
buildQueries();


const Workers = [{
        src: path.join(__dirname, '.', 'src', '.', 'IDBObjectStores', '.', 'assets'),
        globalName: '$objectStore'
    },
    {
        src: path.join(__dirname, '.', 'src', '.', 'IDB'),
        globalName: '$idb'
    },
];

Workers.forEach(({ src, globalName }) => setDirAsIndex(src, globalName));

function setDirAsIndex(src, globalName) {

    const dest = path.join(__dirname, '.', 'src', '.', 'workers', '.', 'assets');

    function initBuild() {
        return new Promise(resolve => {
            const files = readdirSync(src).filter(e => {
                if (/\.js$/.test(e) && 'index.js' !== e)
                    return e;
            }).map(e => {

                const key = e.replace('__', '').replace(/\.js$/, '');
                const val = `require('./${e}')`;

                return `exports.${key} = ${val};`;
            }).join('\n');

            let content = '/* jshint esversion : 6 */\n';
            //content += 'require("@babel/register");\nrequire("@babel/polyfill/noConflict");\nrequire("@babel/plugin-transform-runtime");\n';
            content += `${files}\nself.${globalName} = exports;`;



            try {
                unlinkSync(`${src}/index.js`);
            } catch (e) {
                setTimeout(Function.prototype, 10000);
            }

            writeFile(`${src}/index.js`, content, function(err) {
                resolve();
            });
        });
    }

    const entries = path.join(src, '.', 'index.js');


    function build() {
        return browserify({ entries })
            .transform(babel)
            .bundle()
            .on('error', err => console.error(err))
            .pipe(source(`${globalName}.js`))
            .pipe(buffer())
            .pipe(gulp.dest(dest))
    }



    gulp.task(`build_${globalName}`, gulp.series(initBuild, build), () => Promise.resolve());
}


function buildXhr() {
    return browserify({
            entries: `${__dirname}/src/axios.js`
        })
        .transform(babel)
        .bundle()
        .on('error', err => console.error(err))
        .pipe(source('$axios.js'))
        .pipe(buffer())
        .pipe(gulp.dest(`${__dirname}/src/workers/assets`));
}

function buildCron() {
    return browserify({
            entries: `${__dirname}/src/cron.js`
        })
        .transform(babel)
        .bundle()
        .on('error', err => console.error(err))
        .pipe(source('$cron.js'))
        .pipe(buffer())
        .pipe(gulp.dest(`${__dirname}/src/workers/assets`));
}


function buildAssets() {
    const src = path.join(__dirname, '.', 'node_modules', '.', 'app-utilities', '.', 'dist', '.', 'app.js');
    const dest = path.join(__dirname, '.', 'src', '.', 'workers', '.', 'assets');

    return gulp.src(src).pipe(gulp.dest(dest));
}


function clear() {
    return Promise.all([
        del(path.join(__dirname, '.', 'src', '.', 'workers', '.', '__$idb.js')),
        del(path.join(__dirname, '.', 'src', '.', 'workers', '.', '__$objectStore.js')),
        del(path.join(__dirname, '.', 'src', '.', 'workers', '.', '__$queries.js')),
        del(path.join(__dirname, '.', '__index.js'))
    ]);
}







gulp.task('default',
    gulp.series(
        buildWorkerPolyfill,
        buildIDBShim,
        buildAssets,
        'build_$idb',
        'build_$objectStore',
        `build_$queries`,
        buildXhr,
        buildCron,
        buildIndex,
        clear
    )
);