/* jshint esversion : 6 */
/* jshint ignore : start */
const { create, del, init } = require('../../object-store');
const webWorkers = require('../../web-workers');


class ObjectStore {
    constructor({ dbName, storeName, alias, index, autoIncrement = false, keyPath = [], httpDataSource, clearBeforeUpdate = false, onlineFirst = true, cronExpression }) {
        this.dbName = dbName;
        this.storeName = storeName;
        this.alias = alias;
        this.indexes = index;
        this.autoIncrement = autoIncrement;
        this.keyPath = keyPath;
        //
        this.clearBeforeUpdate = clearBeforeUpdate;
        //
        this.httpDataSource = httpDataSource;
        this.onlineFirst = onlineFirst;
        this.cronExpression = cronExpression;
        //
        this.ready = $q.defer();
        this.dataReady = $q.defer();
        //
        this.webWorkers = webWorkers;
        this.boot();
    }

    get config() {
        return {
            storeName: this.storeName,
            autoIncrement: this.autoIncrement,
            keyPath: this.keyPath,
            indexes: this.indexes
        };
    }

    create() {
        try {
            return webWorkers.wq({ path: 'objectStore', req: { cmd: 'create', dbName: this.dbName, config: this.config } });
        } catch (err) {
            return create(this.dbName, this.config);
        }
    }

    init() {
        try {
            return webWorkers.wq({ path: 'objectStore', req: { cmd: 'init', dbName: this.dbName, config: this.config } });
        } catch (err) {
            return init(this.dbName, this.config);
        }

    }

    del() {
        try {
            return webWorkers.wq({ path: 'objectStore', req: { cmd: 'del', dbName: this.dbName, config: this.config } });
        } catch (err) {
            return del(this.dbName, this.storeName);
        }
    }

    async boot() {
        try {
            await this.init();
            this.ready.resolve();
        } catch (err) {
            throw err;
        }
    }


    getModel() {
        return new Promise(async resolve => {
            if (this.onlineFirst) {
                try {
                    this.syncWithServer({ onHttpData(m) { return resolve(m) } });
                } catch (err) {
                    const m = await this.getAll();
                    resolve(m);
                }
            } else {
                const m = await this.getAll();
                resolve(m);
            }

        });
    }
}

module.exports = ObjectStore;