/* jshint esversion : 6 */
/* jshint ignore : start */
const { xhr } = require('../../network/xhr');
//
const ObjectStoreQueries = require('./ObjectStoreQueries');
class ObjectStoreSync extends ObjectStoreQueries {
    constructor(o) {
        super(o);
    }

    /* salva info sincronizzazione in localStorage */
    saveSyncInfo(o) {
        saveDataSyncInfos(this.dbName, this.storeName, o);
    }

    /* accedi a info sincronizzaione */
    get syncInfo() {
        return getStoreSyncInfo(this.dbName, this.storeName);
    }


    getRemoteData() {
        try {
            return this.webWorkers.wq({ path: 'xhr', req: { url: this.httpDataSource } });
        } catch (err) {
            return xhr(this.httpDataSource);
        }
    }



    /* sincronizza con server */
    async syncWithServer(eventsCallback, notify) {
        //
        if (!eventsCallback) eventsCallback = {};
        //
        const today = toDay();
        const { lastDataSync } = this.syncInfo;
        const first = !lastDataSync || lastDataSync.day === today;
        //
        //
        const response = ({ step, msg, error = false, httpData }) => {
            const o = { main: false, storeName: this.storeName, dbName: this.dbName, step, msg, alias: this.alias, error };
            if (notify) notify(o);
            switch (step) {
                case 'error':
                    if (eventsCallback.onError) eventsCallback.onError(msg);
                    break;
                case 'init':
                    if (eventsCallback.onInit) eventsCallback.onInit(o);
                    break;
                case 'sync':
                    if (eventsCallback.onSync) eventsCallback.onSync(o);
                    if (eventsCallback.onHttpData) eventsCallback.onHttpData(httpData);
                    break;
                case 'update':
                    if (eventsCallback.onUpdate) eventsCallback.onUpdate(o);
                    break;
                case 'end':
                    if (eventsCallback.onEnd) eventsCallback.onEnd(o);
                    break;

            }
            return o;
        };
        const fail = err => {
            error = true;
            msg = `Rejected: ${first ? 'first' : ''} sync for objectStore "${this.storeName}" not completed. Error: ${err}.`;
            response({ step: 'error', msg, error });
            throw err;
        };


        response({ step: 'init' });

        let msg, error = false;

        /* attendi dati dal server */
        const httpData = await this.getRemoteData().catch(fail);
        this.dataReady.resolve(httpData);
        response({ step: 'sync', httpData });


        /* attendi pulizia store prima di aggiornare se impostata opzione "clearBeforeUpdate" */
        if (this.clearBeforeUpdate) {
            response({ step: 'clear' });
            await this.clear().catch(fail);
        }



        /* attendi aggiornamento store */
        response({ step: 'update' });
        if (httpData && httpData.length)
            await this.bulkUpdate(httpData).catch(fail);

        msg = `Resolved: ${first ? 'first' : ''} sync for objectStore "${this.storeName}" completed succesfly`;



        const o = {};
        o[first ? 'firstDataSync' : 'lastDataSync'] = { day: toDay(), time: Now() };
        this.saveSyncInfo(o);

        return response({ step: 'end', msg, error });
    }


    /* sincronizza dati a determinato evento */
    async syncDataOnEvent({ target, event }, result, callback) {
        (target || window).addEventListener(event, async() => {
            try {
                const msg = await this.syncWithServer({ onHttpData(data) { return result.httpRequestEnd(data); } }, callback);
                if (result) result.allComplete(msg);
                return msg;
            } catch (e) {
                if (result) result.allFail(e);
                throw e;
            }
        });
    }

    /* sincronizza al primo accesso del giorno */
    async syncDataOnFirstAccess(result, callback) {
        const successMsg = `Resolved: daily sync for objectStore "${this.storeName}" completed succesfly`;
        const today = toDay();

        const { lastDataSync } = this.syncInfo;
        const first = !lastDataSync || lastDataSync.day !== today;
        if (!first) {
            const data = await this.getAll();
            this.dataReady.resolve(data);
            return successMsg;
        }

        try {
            const msg = await this.syncWithServer({ onHttpData(data) { return result.httpRequestEnd(data); } }, callback);
            //
            if (result) result[!first ? 'allComplete' : 'firstSyncEnd'](msg);
            return successMsg;
        } catch (err) {
            result.syncFail(err);
            if (first) result.firstSyncFail(err);
            throw err;
        }


    }

    /* sincronizza dati (dispatcher per varie modalita di sync) */
    async syncData(opt, callback) {

        const response = ({ step, msg, error = false }) => {
            const o = { main: false, storeName: this.storeName, dbName: this.dbName, step, msg, alias: this.alias, error };
            if (callback) callback(o);
            return o;
        };

        response({ step: 'init' });

        const result = syncResult();

        // opt o proprieta "httpDataSource" sono obbligatori
        if (!opt && !this.httpDataSource) {
            result.allComplete(`Resolved: no syncOptions or httpDataSource setted for objectStore "${this.storeName}"`);
            response({ step: 'end' });
            return result;
        }

        if (!opt) opt = {};

        opt.taskName = this.storeName;
        opt.httpParams = this.httpDataSource;


        let localData;
        try {
            localData = await this.getAll();
        } catch (err) {
            throw err;
        }


        if (!localData || !localData.length) {
            try {
                await this.syncWithServer({ onHttpData(data) { return result.httpRequestEnd(data); } }, callback)
            } catch (err) {
                result.syncFail(err);
                result.firstSyncFail(err);
                throw err;
            }
        } else
            result.firstSyncEnd(`Resolved: first sync for objectStore "${this.storeName}" completed succesfly`);


        if (opt.updateOnFirstAccess || this.onlineFirst)
            await this.syncDataOnFirstAccess(result, callback);
        //
        if (opt.updateOnEvent)
            this.syncDataOnEvent(opt.updateOnEvent, result, callback);
        //
        if (opt.onTime) {
            this.syncXhrData(opt, async(err, data) => {
                if (err) return result.syncFail(err);


                try {
                    await this.bulkUpdate(data);
                    result.syncEnd(`Resolved: sync scheduled at "${opt.onTime}" for objectStore "${this.storeName}" completed succesfly`);
                    //
                    this.saveSyncInfo({ lastDataSync: { day: toDay(), time: Now() } });
                } catch (e) {
                    result.syncFail(e);
                    throw e;
                }


            });
        }
        //
        response({ step: 'end' });
        return result;
    }

}
module.exports = ObjectStoreSync;



function syncResult() {

    const syncComplete = $q.defer();
    const firstSyncComplete = $q.defer();
    const httpRequestComplete = $q.defer();

    class DataSyncResult {
        constructor() {}


        get httpRequestComplete() {
            return httpRequestComplete.promise;
        }

        get syncComplete() {
            return syncComplete.promise;
        }

        get firstSyncComplete() {
            return firstSyncComplete.promise;
        }

        get complete() {
            return Promise.race([this.httpRequestComplete, this.firstSyncComplete, this.syncComplete]);
        }

        //
        httpRequestFail(e) {
            httpRequestComplete.reject(e);
        }
        httpRequestEnd(e) {
            httpRequestComplete.resolve(e);
        }

        //
        firstSyncFail(e) {
            firstSyncComplete.reject(e);
        }

        firstSyncEnd(e) {
            firstSyncComplete.resolve(e);
        }

        //
        syncFail(e) {
            syncComplete.reject(e);
        }

        syncEnd(a) {
            syncComplete.resolve(a);
        }

        //
        allFail(e) {
            this.httpRequestFail(e);
            this.firstSyncFail(e);
            this.syncEnd(e);
        }

        allComplete(a) {
            this.httpRequestEnd(a);
            this.firstSyncEnd(a);
            this.syncEnd(a);
        }



    }

    return new DataSyncResult();
}



function saveDataSyncInfos(dbName, storeName, { firstDataSync, lastDataSync }) {
    const syncInfo = $localStorage.getItem('IDBDataSyncInfos') || {};

    if (!syncInfo[dbName]) syncInfo[dbName] = {};
    if (!syncInfo[dbName][storeName]) syncInfo[dbName][storeName] = {};


    if (!syncInfo[dbName][storeName].firstDataSync) {
        syncInfo[dbName][storeName].firstDataSync = firstDataSync;
        syncInfo[dbName][storeName].lastDataSync = firstDataSync;
    }
    if (lastDataSync) syncInfo[dbName][storeName].lastDataSync = lastDataSync;

    $localStorage.setItem('IDBDataSyncInfos', syncInfo);
}


function getStoreSyncInfo(dbName, storeName) {
    let info = $localStorage.getItem('IDBDataSyncInfos');
    if (!info) {
        info = {};
        $localStorage.setItem('IDBDataSyncInfos', {});
    }

    let dbInfo = info[dbName];
    if (!dbInfo) {
        dbInfo = info[dbName] = {};
        $localStorage.setItem('IDBDataSyncInfos', info);
    }

    let storeInfo = info[dbName][storeName];
    if (!storeInfo) {
        storeInfo = info[dbName][storeName] = {
            firstDataSync: { day: null, time: null },
            lastDataSync: { day: null, time: null }
        };
        $localStorage.setItem('IDBDataSyncInfos', info);
    }

    return storeInfo;
}