/* jshint esversion : 6 */


exports.open = function IDBOpenRequest({ dbName, version, onsuccess, onupgradeneeded, onerror, onabort, onclose }) {
    return new Promise(resolve => {

        let idb;

        const IDBOpenRequest = isNaN(version) ? indexedDB.open(dbName) : indexedDB.open(dbName, version);

        IDBOpenRequest.onsuccess = ev => {
            idb = ev.target.result;
            resolve(idb);
            const version = idb.version;
            if (onsuccess) onsuccess({ version, idb });
        };

        IDBOpenRequest.onupgradeneeded = (ev) => {
            idb = ev.target.result;
            const version = idb.version;
            if (onupgradeneeded) onupgradeneeded({ version, idb });
        };


        IDBOpenRequest.onerror = e => {
            if (onerror) onerror(e);
            throw e;
        };

        IDBOpenRequest.onabort = e => {
            if (onabort) onabort(e);
            throw e;
        };


        IDBOpenRequest.onclose = e => {
            if (onclose) onclose(e);
            throw e;
        };

    });
};


exports.close = function IDBCloseConnection(idb) {
    return new Promise(resolve => {
        try {
            idb.close();
        } catch (e) {
            void 0;
        }
        resolve();
    });
};