/* jshint esversion : 6 */
/* jshint ignore : start */

const ObjectStore = require('./ObjectStore');
const connection = require('./connection');


class IDB {

    constructor({ dbName, alias, objectStores = [] }) {
        this.dbName = dbName;
        this.alias = alias;
        this.config = { stores };
        this.stores = {};
        //
        this.boot();
        this.ready = $q.defer();
    }


    async boot(callback) {
        try {
            await connection.open({ dbName: this.dbName });
            await this.configStores(callback);
            await this.syncAllStores(callback);
            this.ready.resolve();

            return this.ready.promise;
        } catch (err) {
            throw err;
        }

    }


    async configStores(callback) {
        const queue = this.config.stores.map(async o => {

            const notify = (msg, err) => {
                if (callback) callback({ dbName: this.dbName, storeName: o.storeName, msg, err });
            };

            try {
                o.dbName = this.dbName;
                const store = new ObjectStore(o);
                notify('config start');
                await store.ready.promise;
                notify('config end');
                this.stores[o.storeName] = store;
                return store;
            } catch (err) {
                notify('config error', err);
                throw err;
            }
        });
        return Promise.all(queue);
    }


    async objectStore(storeName) {
        const store = this.stores[storeName];

        if (!store) {
            const storeConfig = this.config.stores[storeName];
            //
            if (!storeConfig) throw `objectStore '${storeName}' not exists in db '${this.dbName}'`;
            //
            storeConfig.dbName = this.dbName;
            const store = new ObjectStore(storeConfig);
            await store.ready.promise;
            this.stores[o.storeName] = store;
            return store;
        } else
            return store;
    }


    async syncStore(storeName, callback) {
        try {
            const store = await this.objectStore(storeName);
            await store.syncData(callback);
            return true;
        } catch (err) {
            throw err;
        }
    }


    async syncAllStores(callback) {
        try {
            const queue = this.config.stores
                .filter(e => {
                    if (e.httpDataSource)
                        return e;
                })
                .map(e => this.syncStore(e.storeName, callback));

            await Promise.all(queue);
            return true;
        } catch (err) {
            throw err;
        }
    }

}

module.exports = IDB;