/* jshint esversion : 6 */
/* jshint ignore : start */

require('./helpers');
require('./storages/local-storage');
const { xhr } = require('./network/xhr');
const IDB = require('./idb');
const webWorker = require('./web-workers');
//
//
//
const SCHEMAS = {};
const READY = $q.defer();


class AppIDB {

    get ready() {
        return READY.promise;
    }

    set workersPath(x) {
        webWorker.workersPath = x;
    }

    saveConfig(params) {
        const config = params.reduce((curr, next) => {
            curr[next.dbName] = next;
            return curr;
        }, {});

        $localStorage.setItem('IDBConfig', config);
    }

    getLocalConfig() {
        const config = $localStorage.getItem('IDBConfig');
        if (!config) return null;
        return Object.values(config);
    }

    loadConfig(httpParams) {
        return new Promise(async resolve => {

            let dbsParamas;
            try {
                dbsParamas = await xhr(httpParams);
            } catch (err) {
                dbsParamas = this.getLocalConfig();
                if (!dbsParamas) throw err;
            }
            this.saveConfig(dbsParamas);

            resolve(dbsParamas);
        });
    }



    async config(opt, callback) {
        if (opt.syncConfig) {
            await this.syncConfig(opt.syncConfig);
        }

        if (opt.syncData) {
            const syncFn = opt => this[opt.dbNames === '*' ? 'syncAllData' : 'syncData'](opt, callback);

            if (!isArray(opt.syncData))
                await syncFn(opt.syncData);
            else {

                const queue = opt.syncData.map(syncFn);
                await Promise.all(queue);
            }
        }

        READY.resolve();
        return this.ready;
    }

    async syncConfig(httpParams) {
        const dbsParamas = await this.loadConfig(httpParams).catch(e => { throw e; });
        return this.setDbs(dbsParamas);
    }

    async syncAllData(opt, callback) {
        await this.ready;
        opt.dbNames = Object.keys(SCHEMAS);
        return this.syncData(opt, callback);
    }


    async syncData(o, callback) {
        const { dbNames } = o;
        await this.ready;
        const taskQueue = dbNames.map(async e => {
            const db = SCHEMAS[e];

            notify(db, false, callback);
            await db.syncAllStores(o, callback);
            notify(db, true, callback);

            return Promise.resolve();
        });

        return Promise.all(taskQueue);
    }



    newSchema(dbName, opt) {
        if (!SCHEMAS[dbName]) {
            SCHEMAS[dbName] = IDB(opt);
        }
        return SCHEMAS[dbName];
    }


    async setDb(opt) {
        try {
            const db = this.newSchema(opt);
            return db.ready.promise;
        } catch (err) {
            throw err;
        }
    }


    async setDbs(dbsParamas) {
        if (!isArray(dbsParamas))
            return this.setDb(dbsParamas);

        const queue = dbsParamas.map(e => this.setDb(e));
        return Promise.all(queue);
    }


    useDb(dbName) {
        return SCHEMAS[dbName];
    }


    async forceDataSync(callback) {
        await this.ready;
        const taskQueue = Object.values(SCHEMAS).map(async db => {

            notify(db, false, callback);
            const queue = Object.values(db._stores).map(store => store.syncWithServer(null, null, callback));
            await Promise.all(queue).catch(err => console.log(err));

            notify(db, true, callback);

            return Promise.resolve();
        });

        return Promise.all(taskQueue).catch(err => console.log(err));
    }
}


function notify({ dbName, alias }, complete, callback) {
    const o = { main: true, step: 'dbSync', dbName, alias, complete };
    if (callback) callback(o);
    return o;
}



global.$appIDB = new AppIDB();