/* jshint esversion : 6 */
/* jshint ignore : start */


module.exports = connection => {


    const idb_indexes = require('./indexes')(connection);
    const idb_keyPath = require('./key-path')(connection);

    return function initStore(dbName, { storeName, autoIncrement, keyPath, indexes = [] }) {
        return new Promise(resolve => {
            connection.open({
                dbName,
                onsuccess: async({ idb }) => {
                    try {
                        let store;
                        if (!idb.objectStoreNames.contains(storeName)) {
                            store = await createStore(dbName, { storeName, autoIncrement, keyPath, indexes });
                        } else {

                            const objectStore = idb.transaction(storeName);

                            const [cfg_indexes, cfg_keyPath] = await Promise.all([
                                idb_indexes.check(objectStore, indexes),
                                idb_keyPath.check(objectStore, keyPath)
                            ]);

                            if (cfg_indexes.toModify || cfg_keyPath.toModify) {
                                await deleteStore(dbName, storeName);
                                await connection.close(idb);
                                store = await createStore(dbName, { storeName, autoIncrement, keyPath, indexes });
                            } else
                                store = idb.objectStore(storeName);

                            resolve(store);
                        }

                    } catch (err) {
                        connection.close(idb);
                        throw err;
                    }
                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });
        });

    };
};