/* jshint esversion : 6 */
/* jshint ignore : start */


module.exports = function({ connection, IDBKeyRange, IDBTransaction, Helpers }) {

    return function Cursor({ dbName, storeNames, query = {} }, notify) {

        return new Promise(resolve => {

            connection.open({
                dbName,
                onsuccess: ({ idb }) => {

                    const catchFn = e => {
                        connection.close(idb);
                        throw e;
                    };

                    let KeyRange;
                    if (IDBKeyRange.test(key)) KeyRange = IDBKeyRange.set(key.limit, key.range);

                    try {
                        // SOLO PER 'SELECT'
                        const SELECTION = [];
                        //
                        const tx = IDBTransaction({ idb, storeNames, flag: "readwrite" });
                        //
                        const { key, direction } = query;
                        const storeTx = (key && key.onIndex) ? tx.objectStore(storeNames).index(key.onIndex) : tx.objectStore(storeNames);
                        //
                        const cursor = k ? (direction ? storeTx.openCursor(k, direction) : storeTx.openCursor(KeyRange)) : storeTx.openCursor();
                        //
                        cursor.onsuccess = ev => {
                            const cursor = ev.target.result;
                            if (cursor) {
                                //
                                const { value, direction, key, primaryKey, state = "progress" } = cursor;
                                //
                                if (query.where) query.searchCondition = Helpers.searchCondition(value, query.where);
                                switch (query.cmd) {
                                    case 'update':
                                        Update(cursor, query).catch(catchFn).then(notify);
                                        break;

                                    case 'delete':
                                        Delete(cursor, query).catch(catchFn).then(notify);
                                        break;

                                    case 'select':
                                        const v = Select(cursor, query);
                                        if (v) SELECTION.push(v);
                                        break;

                                    case 'sync':
                                    default:
                                        if (notify) notify({ value, direction, key, primaryKey, state });
                                        break;
                                }
                                //
                                cursor.continue();
                            } else if (notify)
                                notify({ state: 'end' });
                        };

                        tx.oncomplete = async ev => {
                            await connection.close(idb);
                            switch (query.cmd) {
                                case 'select':
                                    const result = Helpers.transformSelection(SELECTION);
                                    resolve({ state: 'complete', result });
                                    break;
                                default:
                                    resolve({ state: 'complete' });
                                    break;
                            }

                        };

                        tx.onerror = catchFn;

                        tx.onabort = catchFn;


                    } catch (e) {
                        catchFn(e);
                    }





                    const getKeyPathValue = (o, kp) => JSON.stringify(Object.values(JSON.parse(JSON.stringify(o, kp))));
                    //

                    function Update(cursor, { data: newValue, searchCondition, onKeyUpdate }) {

                        const { value: cursorValue, direction, key, primaryKey, source: { keyPath } } = cursor;

                        const oldKeyValues = getKeyPathValue(cursorValue, keyPath);
                        const newKeyValues = getKeyPathValue(newValue, keyPath);
                        const updateByKeyValues = onKeyUpdate && Helpers.equals(oldKeyValues, newKeyValues);

                        return new Promise(resolve => {

                            if (searchCondition || updateByKeyValues) {

                                const req = cursor.update(newValue);
                                req.onsuccess = () => {
                                    resolve({
                                        oldValue: cursorValue,
                                        newValue,
                                        direction,
                                        key,
                                        primaryKey,
                                        state: "update",
                                        msg: "progress"
                                    });
                                };
                                req.onerror = () => {
                                    Delete(cursor, { searchCondition, onKeyUpdate })
                                        .catch(e => { throw e; })
                                        .then(resolve);
                                };
                            }
                        });
                    }



                    function Delete(cursor, { data: newValue, searchCondition, onKeyUpdate }) {

                        const { value: cursorValue, direction, key, primaryKey, source: { keyPath } } = cursor;

                        let oldKeyValues, newKeyValues, updateByKeyValues;

                        if (newValue && onKeyUpdate) {
                            oldKeyValues = getKeyPathValue(cursorValue, keyPath);
                            newKeyValues = getKeyPathValue(newValue, keyPath);
                            updateByKeyValues = onKeyUpdate && Helpers.equals(oldKeyValues, newKeyValues);
                        }

                        return new Promise(resolve => {

                            if (searchCondition || updateByKeyValues) {
                                const req = cursor.delete();
                                req.onsuccess = () => {
                                    resolve({
                                        deletedValue: cursorValue,
                                        direction,
                                        key,
                                        primaryKey,
                                        state: "delete",
                                        msg: "progress"
                                    });
                                };
                                req.onerror = ev => { throw ev; };
                            }
                        });
                    }



                    function Select(cursor, { data: newValue, searchCondition }) {

                        const { value: cursorValue, source: { keyPath } } = cursor;

                        let oldKeyValues, newKeyValues;

                        if (newValue) {
                            oldKeyValues = getKeyPathValue(cursorValue, keyPath);
                            newKeyValues = getKeyPathValue(newValue, keyPath);
                        }

                        if (searchCondition || Helpers.equals(oldKeyValues, newKeyValues)) {
                            return cursorValue;
                        } else
                            return false;

                    }


                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });
        });
    };


    /* jshint ignore : end */
};