/* jshint esversion : 6 */
/* jshint ignore : start */

module.exports = function({ connection, IDBKeyRange, IDBTransaction, Helpers }) {
    return function Delete({ dbName, storeNames, query: { key } }) {

        return new Promise(resolve => {

            connection.open({
                dbName,
                onsuccess: function({ idb }) {

                    try {
                        let KeyRange;
                        if (IDBKeyRange.test(key)) KeyRange = IDBKeyRange.set(key.limit, key.range);

                        const tx = IDBTransaction({ idb, storeNames, flag: "readwrite" });
                        const storeTx = tx.objectStore(storeNames);
                        const del = KeyRange ? storeTx.delete(KeyRange) : storeTx.clear();

                        let result;
                        //
                        tx.oncomplete = ev => {
                            resolve(result);
                            connection.close(idb);
                        };
                        tx.onerror = ev => {
                            connection.close(idb);
                            throw ev;
                        };
                        tx.onabort = ev => {
                            connection.close(idb);
                            throw ev;
                        };
                        //
                        del.onsuccess = ev => result = ev.target.result;
                        del.onerror = async ev => {
                            await connection.close(idb);
                            throw ev;
                        };

                    } catch (e) {
                        connection.close(idb);
                        throw e;
                    }

                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });

        });

    };
};