/* jshint esversion : 6 */
/* jshint ignore : start */

module.exports = function({ connection, IDBKeyRange, IDBTransaction, Helpers }) {

    return function GetAll({ dbName, storeNames, query: { cmd = 'getAll', key } }) {
        return new Promise(resolve => {
            connection.open({
                dbName,
                onsuccess: function({ idb }) {
                    try {
                        let KeyRange, result;
                        if (IDBKeyRange.test(key)) KeyRange = IDBKeyRange.set(key.limit, key.range);
                        //
                        const tx = IDBTransaction({ idb, storeNames, flag: "readonly" });
                        tx.oncomplete = ev => {
                            if (!result) result = ev.target.result;
                            //
                            const dt = Helpers.transformSelection(result);
                            resolve(dt);
                            connection.close(idb);
                        };
                        tx.onerror = ev => {
                            connection.close(idb);
                            throw ev;
                        };
                        tx.onabort = ev => {
                            connection.close(idb);
                            throw ev;
                        };
                        //
                        //
                        const count = query.count;
                        const storeTx = (key && key.onIndex) ? tx.objectStore(storeNames).index(key.onIndex) : tx.objectStore(storeNames);
                        const getAll = k ? (count ? storeTx[cmd](k, count) : storeTx[cmd](KeyRange)) : storeTx[cmd]();
                        getAll.onsuccess = ev => result = ev.target.result;
                        getAll.onerror = async ev => {
                            await connection.close(idb);
                            throw ev;
                        };

                    } catch (e) {
                        connection.close(idb);
                        throw e;
                    }

                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });
        });
    }
};