/* jshint esversion : 6 */


const $q = self.$q = require('q');
const moment = self.moment = require('moment/min/moment-with-locales');
require('app-polyfill');
require('app_local_storage');



const IDB = require('./IDB');
const IDBCron = require('./IDBCron');
const { isIDBSupported } = require('app-utilities');





/* jshint ignore : start */

class AppIDBConfig extends IDBCron {
    constructor() {
        super();
    }

    saveConfig(params) {
        const config = params.reduce((curr, next) => {
            curr[next.dbName] = next;
            return curr;
        }, {});

        $localStorage.setItem('IDBConfig', config);
    }

    getLocalConfig() {
        const config = $localStorage.getItem('IDBConfig');

        if (!config) return null;

        return Object.values(config);
    }


    loadConfig(httpParams) {
        return new Promise(async resolve => {

            const dbsParamas = await this.xhr(httpParams).catch(e => {
                const dbsParamas = this.getLocalConfig();

                if (!dbsParamas) throw e;

                resolve(dbsParamas);
            });
            this.saveConfig(dbsParamas);

            resolve(dbsParamas);
        });
    }
}




const SCHEMAS = {};
const CONFIGS = {};
const READY = $q.defer();
const FIRTS_DATA_SYNC = $q.defer();

class AppIDB extends AppIDBConfig {

    constructor() {
        super();
        this.debug = false;
    }

    get ready() {
        return READY.promise;
    }



    async config(opt, callback) {
        //await this.installWorkers();

        if (opt.syncConfig) {
            await this.syncConfig(opt.syncConfig);
            READY.resolve();
        }
        if (opt.syncData) {

            const syncFn = opt => this[opt.dbNames === '*' ? 'syncAllData' : 'syncData'](opt, callback);

            if (!Array.isArray(opt.syncData))
                await syncFn(opt.syncData);
            else {

                const queue = opt.syncData.map(syncFn);
                await Promise.all(queue);
            }
            FIRTS_DATA_SYNC.resolve();
        }

        return Promise.resolve();
    }
    async syncConfig(httpParams) {
        const dbsParamas = await this.loadConfig(httpParams).catch(e => { throw e; });
        return this.setDbs(dbsParamas);
    }


    get firstDataSync() {
        return FIRTS_DATA_SYNC.promise;
    }

    newSchema(dbName, opt) {
        if (!SCHEMAS[dbName]) {
            SCHEMAS[dbName] = IDB(dbName, opt.alias);
            CONFIGS[dbName] = opt;
        }
        return SCHEMAS[dbName];
    }

    /** 
     * @param {String} dbName - name of db
     * @param {Object} opt - config options
     * @returns {Promise} promise
     */

    async setDb(dbName, opt) {
        await isIDBSupported().catch(err => { db.ready = 'n'; throw err; });


        const db = this.newSchema(dbName, opt);

        if (!opt) {
            await db.connect();
            db.ready = 'y';
        } else if (opt.objectStores) {
            await db.createObjectStores(opt.objectStores);
            db.ready = 'y';
        } else {
            db.ready = 'n';
        }

        return db.ready;


    }


    async setDbs(dbsParamas) {
        if (!Array.isArray(dbsParamas))
            return this.setDb(dbsParamas.dbName, dbsParamas);

        const queue = dbsParamas.map(e => this.setDb(e.dbName, e));

        return Promise.all(queue);
    }


    useDb(dbName) {
        return SCHEMAS[dbName];
    }

    async syncAllData(opt, callback) {
        await this.ready;
        opt.dbNames = Object.keys(SCHEMAS);
        return this.syncData(opt, callback);
    }


    async syncData(o, callback) {
        const { dbNames } = o;
        await this.ready;
        const taskQueue = dbNames.map(async e => {
            const db = SCHEMAS[e];

            notify(db, false, callback);
            await db.syncAllData(o, callback);
            notify(db, true, callback);

            return Promise.resolve();
        });

        return Promise.all(taskQueue);
    }


    async forceDataSync(callback) {

        await this.ready;
        const taskQueue = Object.values(SCHEMAS).map(async db => {

            notify(db, false, callback);
            const queue = Object.values(db._stores).map(store => store.syncWithServer(null, null, callback));
            await Promise.all(queue).catch(err => console.log(err));

            notify(db, true, callback);

            return Promise.resolve();
        });


        return Promise.all(taskQueue).catch(err => console.log(err));

    }

}

function notify({ dbName, alias }, complete, callback) {
    const o = { main: true, step: 'dbSync', dbName, alias, complete };
    if (callback) callback(o);
    return o;
}

/* jshint ignore : end */


self.$appIDB = new AppIDB();