/* jshint esversion : 6 */

module.exports = IDBOpenRequest;



function IDBOpenRequest({ dbName, version, onsuccess, onupgradeneeded, onerror, onabort, onclose }) {
    return new Promise(resolve => {

        let idb;

        const _DBOpenRequest = IDBOpenDBRequest({ dbName, version });

        _DBOpenRequest.onsuccess = ev => {
            idb = ev.target.result;
            const version = idb.version;
            if (onsuccess) onsuccess({ version, idb });
            resolve(idb);
        };

        _DBOpenRequest.onupgradeneeded = (ev) => {
            idb = ev.target.result;
            const version = idb.version;
            if (onupgradeneeded) onupgradeneeded({ version, idb });
        };


        _DBOpenRequest.onerror = e => {
            if (onerror) onerror(e);
            resolve(Promise.reject(e));
        };

        _DBOpenRequest.onabort = e => {
            if (onabort) onabort(e);
            resolve(Promise.reject(e));
        };


        _DBOpenRequest.onclose = e => {
            if (onclose) onclose(e);
            resolve(Promise.reject(e));
        };

    });


    function IDBOpenDBRequest({ dbName, version }) {
        if (isNaN(version))
            return indexedDB.open(dbName);

        return indexedDB.open(dbName, version);
    }
}