/* jshint esversion : 6 */
const IDBObjectStoreQueries = require('./IDBObjectStoreQueries');
const IDBObjectStoreSync = require('./IDBObjectStoreSync');
const IDBCron = require('../IDBCron');
const initObjectStore = require('./assets/initObjectStore');
const createObjectStore = require('./assets/createObjectStore');
const deleteObjectStore = require('./assets/deleteObjectStore');

const setIndex = require('../IDB/IDBIndex');



module.exports = (dbInstance, opt) => {

    class IDBObjectStore extends IDBCron {
        constructor({ storeName, alias, index, autoincrement = false, keyPath = [], httpDataSource, clearBeforeUpdate = false, onlineFirst = true }) {
            super();
            //
            this.storeName = storeName;
            this.alias = alias || storeName;
            //
            this.indexes = [];
            if (index) {
                if (Array.isArray(index))
                    this.indexes = index.map(setIndex);
                else if ("object" === typeof index)
                    this.indexes.push(setIndex(index));
            }

            this.dbName = dbInstance.dbName;

            this.autoIncrement = autoincrement;

            this.keyPath = keyPath
                .concat(this.indexes.map(e => e.keyPath))
                .reduce((curr, next) => {
                    if (Array.isArray(next))
                        return curr.concat(next);
                    else {
                        curr.push(next);
                        return curr;
                    }
                }, [])
                .reduce((curr, next) => {
                    const i = curr.find(e => e === next);

                    if ("undefined" === typeof i || i == null)
                        curr.push(next);

                    return curr;
                }, []);


            this.httpDataSource = httpDataSource;
            this.onlineFirst = onlineFirst;
            this.clearBeforeUpdate = clearBeforeUpdate;

        }


        checkStoreConfig() {
            return dbInstance.checkStoreConfig(this);
        }



        init(diff) {
            const req = {
                db: { dbName: this.dbName },
                store: {
                    storeName: this.storeName,
                    autoIncrement: this.autoIncrement,
                    keyPath: (this.keyPath || []).merge(),
                    indexes: this.indexes
                },
                diff
            };

            return (this.useWorker ? this.wq({ path: 'objectStores/initObjectStore.js', req, shared: false }) : initObjectStore(req)).catch(e => { throw e; });
        }



        create() {
            const req = {
                db: { dbName: this.dbName },
                store: {
                    storeName: this.storeName,
                    autoIncrement: this.autoIncrement,
                    keyPath: (this.keyPath || []).merge(),
                    indexes: this.indexes
                }
            };

            return (this.useWorker ? this.wq({ path: 'objectStores/createObjectStore.js', req, shared: false }) : createObjectStore(req)).catch(e => { throw e; });
        }

        truncate() {
            const req = {
                db: { dbName: this.dbName },
                store: { storeName: this.storeName }
            };

            return (this.useWorker ? this.wq({ path: 'objectStores/deleteObjectStore.js', req, shared: false }) : deleteObjectStore(req)).catch(e => { throw e; });
        }

        /* jshint ignore: start */
        async alter({ db, dbName }) {

            if (this.useWorker) {
                await this.wq({ path: 'objectStores/initObjectStore.js', req, shared: false }).catch(e => { throw e; });
                return Promise.resolve();
            } else {
                await this.truncate({ db, dbName }).catch(err => { throw err; }).catch(e => { throw e; });
                await this.create({ db, dbName }).catch(err => { throw err; }).catch(e => { throw e; });
                return Promise.resolve();
            }

        }

        /* jshint ignore: end */
    }

    const DBObjectStoreQueries = IDBObjectStoreQueries(IDBObjectStore);

    const DBObjectStoreSync = IDBObjectStoreSync(DBObjectStoreQueries);


    class ObjectStore extends DBObjectStoreSync {
        constructor(opt) {
            super(opt);
        }
    }


    return new ObjectStore(opt);

};