/* jshint esversion : 6 */

const IDBOpenRequest = require('../../IDB/IDBOpenRequest');
const IDBCloseConnection = require('../../IDB/IDBCloseConnection');

module.exports = createObjectStore;


function createObjectStore({ db: { dbName }, store: { storeName, autoIncrement, keyPath, indexes = [] } }) {
    return new Promise(resolve => {
        IDBOpenRequest({
            dbName,
            onsuccess: ({ version, idb }) => {
                version++;

                IDBCloseConnection(idb);

                IDBOpenRequest({
                    dbName,
                    version,
                    onupgradeneeded: ({ idb }) => {
                        try {
                            const store = idb.createObjectStore(storeName, { autoIncrement, keyPath });
                            indexes.forEach(index => {
                                const { name, keyPath, objectParameters } = index;
                                store.createIndex(name, keyPath, objectParameters);
                            });
                        } catch (err) {
                            throw new Error(err);
                        }

                    },
                    onsuccess: () => {
                        resolve({ success: true, notify: null });
                        IDBCloseConnection(idb);
                    },
                    onerror: e => { throw e; },
                    onabort: e => { throw e; },
                    onclose: e => { throw e; }
                });
            }
        });
    });
}