/* jshint esversion : 6 */


const IDBOpenRequest = require('../../IDB/IDBOpenRequest');
const IDBCloseConnection = require('../../IDB/IDBCloseConnection');

const createObjectStore = require('./createObjectStore');
const deleteObjectStore = require('./deleteObjectStore');


module.exports = initObjectStore;



function initObjectStore(e) {
    const { db: { dbName }, store: { storeName }, diff = [] } = e;


    return new Promise(resolve => {

        const resolveFn = idb => {
            IDBCloseConnection(idb);
            resolve({ success: true });
        };
        const throwError = e => { throw e; };


        IDBOpenRequest({
            dbName,
            onsuccess: ({ idb }) => {

                IDBCloseConnection(idb);

                if (!idb.objectStoreNames.contains(storeName)) {
                    createObjectStore(e)
                        .catch(throwError)
                        .then(() => resolveFn(idb));
                } else {
                    if (diff.indexOf(storeName) > -1) {
                        deleteObjectStore(e)
                            .catch(throwError)
                            .then(createObjectStore)
                            .catch(throwError)
                            .then(() => resolveFn(idb));
                    } else {
                        resolveFn(idb);
                    }
                }
            },
            onerror: throwError,
            onabort: throwError,
            onclose: throwError
        });

    });
}