/* jshint esversion : 6 */
/* jshint ignore : start */

require('pseudo-worker/polyfill');
const { useWorker, useSharedWorker, useServiceWorker } = require('app-utilities');
//const $q = require('q');

let workersPath = 'src/workers/';


class IDBWorkers {

    constructor() {
        this.useWorker = useWorker;
        this.useSharedWorker = useSharedWorker;
        //
    }

    set workersPath(x) {
        console.log(`web workers path = ${x}`);
        workersPath = x;
    }

    get workersPath() { return workersPath; }


    async installWorkers() {
        if (!useServiceWorker) return Promise.resolve();
        //


        const scope = `${this.workersPath}ws.js`;
        const reg = await navigator.serviceWorker.register(scope, { scope: this.workersPath }).catch(err => { throw err; });
        console.log(`Registration succeded. Scope is ${reg.scope}`);
        return Promise.resolve();
    }


    worker(x, shared) {
        const w = this.workersPath;
        const url = w.match(/\/$/) ? w : `${w}/`;
        return new(shared && this.useSharedWorker ? SharedWorker : Worker)(`${url}${x}`);
    }





    wq({ path, req, notify, shared }) {
        const q = $q.defer();

        if (!this.useSharedWorker) shared = false;
        const w = this.worker(path, shared);
        const trigger = (shared && this.useSharedWorker ? w.port : w);
        const done = () => {
            if (!shared) trigger.terminate();
        };

        trigger.postMessage(req);
        trigger.onmessage = ({ data }) => {
            if (notify) {
                switch (data.msg) {
                    case 'progress':
                        q.notify(data);
                        break;
                    case 'end':
                        done();
                        q.resolve(data);
                        break;
                    default:
                        done();
                        q.resolve(data);
                        break;
                }
            } else {
                done();
                q.resolve(data);
            }

        };
        trigger.onerror = err => {
            done();
            q.reject(err);
        };
        trigger.onmessageerror = err => {
            done();
            q.reject(err);
        };

        return q.promise;
    }


}

module.exports = IDBWorkers;