/* jshint esversion : 6 */


/* jshint ignore : start */
const _Put = require('./__Insert');
const _CursorUpdate = require('./__CursorUpdate');

module.exports = function({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction }) {


    const Put = _Put({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction });
    const CursorUpdate = _CursorUpdate({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction });


    return async function BulkUpdate({ dbName, storeNames, data, key, direction }, notify) {



        //const first = data.shift();
        let index = -1;
        const mapFn = data => {
            index++;
            return IDBPut({
                dbName,
                storeNames,
                data,
                key,
                direction,
                index
            }, notify);
        };

        //await Promise.all(data.map(mapFn));

        const queue = data.map((next, index) => mapFn(next, index));

        await Promise.all(queue);

        return { msg: 'end' };

    }


    function IDBPut({ dbName, storeNames, data, key, direction, index }, notify) {

        return new Promise(async resolve => {

            const idbTargetResult = await Put({
                dbName,
                storeNames,
                data,
                key,
                cmd: 'put'
            }).catch(async(err) => {
                const cursorResult = await IDBCursor({ dbName, storeNames, data, key, direction, index }, notify);
                switch (cursorResult.msg) {
                    case 'error':
                        throw cursorResult.ev;
                    default:
                        resolve(cursorResult);
                        break;
                }
            });

            const r = { idbTargetResult, data, msg: 'progress', index, storeName: storeNames };

            if (notify) notify(r);

            resolve(r);
        });

    }


    function IDBCursor({ dbName, storeNames, data, key, direction, index }, notify) {

        return new Promise(async resolve => {

            await CursorUpdate({
                    dbName,
                    storeNames,
                    key,
                    direction,
                    query: { data, onKeyUpdate: true }
                }, notify)
                .catch(ev => {
                    resolve({ ev, data, msg: 'error', index });
                });

            resolve({ msg: 'cursorEnd', index });

        });

    }

};



/* jshint ignore : end */