/* jshint esversion : 6 */


module.exports = function({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction }) {
    /* jshint ignore : start */

    return function Count({ dbName, storeNames, key }) {
        return new Promise(resolve => {
            IDBOpenRequest({
                dbName,
                onsuccess: function({ idb }) {

                    let k;
                    if (isKeyRange(key)) k = IDBKeyRange(key.limit, key.range);

                    try {
                        const tx = IDBTransaction({ idb, storeNames, flag: "readonly" });
                        const storeTx = (key && key.onIndex) ? tx.objectStore(storeNames).index(key.onIndex) : tx.objectStore(storeNames);
                        const count = k ? storeTx.count(k) : storeTx.count();

                        let result;
                        //
                        count.onsuccess = ev => result = ev.target.result;

                        tx.oncomplete = ev => {
                            resolve(result);
                            IDBCloseConnection(idb);
                        };
                        tx.onerror = ev => {
                            IDBCloseConnection(idb);
                            throw ev;
                        };
                        tx.onabort = ev => {
                            IDBCloseConnection(idb);
                            throw ev;
                        };
                    } catch (e) {
                        IDBCloseConnection(idb);
                        throw e;
                    }


                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });
        });
    }

    /* jshint ignore : end */
};