/* jshint esversion : 6 */


module.exports = function({ IDBOpenRequest, IDBCloseConnection, isKeyRange, IDBKeyRange, IDBTransaction }) {
    /* jshint ignore : start */

    return function Delete({ dbName, storeNames, key }) {

        return new Promise(resolve => {

            IDBOpenRequest({
                dbName,
                onsuccess: function({ idb }) {

                    try {

                        const k = isKeyRange(key) ? IDBKeyRange(key.limit, key.range) : key;

                        const tx = IDBTransaction({ idb, storeNames, flag: "readwrite" });
                        const storeTx = tx.objectStore(storeNames);
                        const del = k ? storeTx.delete(k) : storeTx.clear();

                        let result;
                        //
                        del.onsuccess = ev => result = ev.target.result;

                        tx.oncomplete = ev => {
                            resolve(result);
                            IDBCloseConnection(idb);
                        };
                        tx.onerror = ev => {
                            IDBCloseConnection(idb);
                            throw ev;
                        };
                        tx.onabort = ev => {
                            IDBCloseConnection(idb);
                            throw ev;
                        };
                    } catch (e) {
                        IDBCloseConnection(idb);
                        throw e;
                    }

                },
                onerror: e => { throw e; },
                onabort: e => { throw e; },
                onclose: e => { throw e; }
            });

        });

    }

    /* jshint ignore : end */

};