/* jshint esversion : 6 */
/* jshint ignore : start */

require('pseudo-worker/polyfill');
const { workerAvailable, sharedWorkerAvailable, serviceWorkerAvailable } = require('./worker-available');


let workersPath = 'workers/';

class Workers {
    constructor() {
        this.workerAvailable = workerAvailable;
        this.sharedWorkerAvailable = sharedWorkerAvailable;
        this.serviceWorkerAvailable = serviceWorkerAvailable;
    }

    set workersPath(x) {
        workersPath = x;
    }

    get workersPath() {
        return workersPath;
    }

    worker(x, shared) {
        const w = this.workersPath;
        const url = w.match(/\/$/) ? w : `${w}/`;
        return new(shared && this.sharedWorkerAvailable ? SharedWorker : Worker)(`${url}${x}`);
    }

    wq({ path, req, notify, shared = sharedWorkerAvailable }) {

        if (this.workerAvailable) {

            try {

                const q = $q.defer();

                if (!this.sharedWorkerAvailable) shared = false;

                // worker
                const w = this.worker(path, shared);
                const trigger = (shared && this.sharedWorkerAvailable ? w.port : w);
                // 
                const done = () => !shared ? trigger.terminate() : null;


                trigger.postMessage(req);
                trigger.onmessage = ({ data }) => {
                    if (data.error) {
                        throw data.msg;
                    }
                    if (notify) {
                        switch (data.msg) {
                            case 'progress':
                                q.notify(data);
                                break;
                            case 'end':
                                done();
                                q.resolve(data);
                                break;
                            default:
                                done();
                                q.resolve(data);
                                break;
                        }
                    } else {
                        done();
                        q.resolve(data);
                    }

                };
                trigger.onerror = err => {
                    done();
                    q.reject(err);
                };
                trigger.onmessageerror = err => {
                    done();
                    q.reject(err);
                };

                return q.promise;

            } catch (err) {
                throw err;
            }
        } else
            throw 'Workers not available';
    }


}


return new Workers();